package com.yizhi.application.controller;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.domain.NewCommentAdmire;
import com.yizhi.application.service.ICommentAdmireService;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.comment.application.vo.domain.NewCommentAdmireVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.feign.remote.RemotePostsClient;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 评论点赞记录表 前端控制器
 * </p>
 *
 * @author yizhi123
 * @since 2019-12-30
 */
@RestController
@RequestMapping("/commentAdmire")
public class CommentAdmireController {

	@Autowired
	private ICommentAdmireService commentAdmireService;

	@Autowired
	private IdGenerator idGenerator;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private RemotePostsClient remotePostsClient;

	@GetMapping("/up")
	public boolean up(@RequestParam(name = "commentId") Long commentId, @RequestParam("bizType") Integer bizType) {
		RequestContext context = ContextHolder.get();
        NewCommentAdmire commentAdmire = new NewCommentAdmire();
        commentAdmire.setId(idGenerator.generate());
        commentAdmire.setAccountId(context.getAccountId());
        commentAdmire.setCommentId(commentId);
        commentAdmire.setBizType(bizType);
        commentAdmire.setCreateById(context.getAccountId());
        commentAdmire.setCreateByName(context.getAccountName());
        commentAdmire.setCreateTime(new Date());
        commentAdmire.setUpdateById(context.getAccountId());
        commentAdmire.setUpdateByName(context.getAccountName());
        commentAdmire.setUpdateTime(new Date());
        commentAdmire.setState(1);
        boolean insert = commentAdmireService.insert(commentAdmire);
        if (insert) {
            updatePostsInfo(bizType, commentId);
        }
        return insert;
	}

	@GetMapping("/down")
    public boolean down(@RequestParam(name = "commentId")Long commentId, @RequestParam("bizType") Integer bizType) {
        RequestContext res = ContextHolder.get();
        NewCommentAdmire commentAdmire = new NewCommentAdmire();
        commentAdmire.setCommentId(commentId);
        commentAdmire.setBizType(bizType);
        commentAdmire.setAccountId(res.getAccountId());
        commentAdmire.setState(1);
        boolean delete = commentAdmireService.delete(new EntityWrapper<>(commentAdmire));
        if (delete){
            updatePostsInfo(bizType, commentId);
        }
        return delete ;
    }

    /**
     * 获取点赞数
     * @param bizType
     * @param taskId
     * @param accountId
     * @return
     */
    @GetMapping("/getNum")
    public Integer getNum(@RequestParam("bizType") Integer bizType,
                          @RequestParam(name = "taskId") Long taskId,
                          @RequestParam(name = "accountId",required = false) Long accountId) {
        RequestContext res = ContextHolder.get();
        NewCommentAdmire commentAdmire = new NewCommentAdmire();
        commentAdmire.setState(Constant.UP);
        commentAdmire.setBizType(bizType);
        commentAdmire.setCommentId(taskId);
        commentAdmire.setAccountId(accountId);

        return commentAdmireService.selectCount(new EntityWrapper<>(commentAdmire));
    }

    /**
     * 获取<业务id,点赞数></>>Map
     * @param bizType
     * @return
     */
    @GetMapping("/getPersonalDataMap")
    public Map<Long, Integer> getPersonalDataMap(@RequestParam("bizType") Integer bizType) {
        return commentAdmireService.getCountsMap(bizType);
    }

    /**
     * 点赞或者取消点赞 更新帖子信息
     * @param bizType
     * @param bizId
     */
    public void updatePostsInfo(Integer bizType, Long bizId) {
        //如果业务类型对帖子的
        if (Constant.TASK_TYPE_POSTS.equals(bizType)) {
            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {
                    NewCommentAdmire commentAdmire = new NewCommentAdmire();
                    commentAdmire.setBizType(bizType);
                    commentAdmire.setCommentId(bizId);
                    commentAdmire.setState(1);
                    //更新帖子新回复、最新评论时间、评论数、回复数字段
                    com.yizhi.forum.application.vo.domain.PostsVo vo = new com.yizhi.forum.application.vo.domain.PostsVo();
                    vo.setId(bizId);
                    vo.setThumbsNum(commentAdmireService.selectCount(new EntityWrapper<>(commentAdmire)));
                    remotePostsClient.updateInfo(vo);
                }
            });
        }
    }
}

