package com.yizhi.application.controller.remote;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.NewReply;
import com.yizhi.comment.application.vo.domain.NewReplyVo;
import com.yizhi.core.application.cache.CacheNamespace;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.util.application.json.JsonUtil;
import com.yizhi.util.application.sensitive.SensitivewordFilter;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.application.service.ICommentService;
import com.yizhi.application.service.IReplyService;
import com.yizhi.comment.application.vo.CommentVo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * <p>
 * 回复 前端控制器
 * </p>
 *
 * @author yizhi123
 * @since 2019-12-30
 */
@RestController(value = "ReplyControllerRemote")
@RequestMapping("/remote/reply")
public class ReplyController {

	@Autowired
	private ICommentService commentService;

	@Autowired
	private IReplyService replyService;

	@Autowired
	private IdGenerator idGenerator;
	
	@Autowired
	private AccountClient accountClient;
	
    @Autowired
    private RedisCache redisCache;
    
	/**
	 * 新增回复评论接口（保存）
	 * @param
	 * @return
	 */
	@PostMapping("/save")
	public Boolean saveReply(@RequestBody NewReply reply){
		RequestContext context = ContextHolder.get();
		Date date = new Date();
		if (reply.getReplyParentId()!=null&&reply.getReplyParentId()!=0) {
			NewReply newReplyVo = replyService.selectById(reply.getReplyParentId());
			if (newReplyVo ==null) {
				return false;
			}
		}else {
			reply.setReplyParentId(0L);
		}
		reply.setId(idGenerator.generate());
		reply.setCreateById(context.getAccountId());
		reply.setCreateByName(context.getAccountName());
		reply.setCreateTime(date);
		if (replyService.insert(reply)){
			commentService.updatePostsInfo(reply.getBizId(),reply.getBizType(),context.getSiteId(),date,true,false);
			return true;
		}
		return false;
	}
	
	@GetMapping("/list")
	public Page<CommentVo> list(@RequestParam(name = "bizId") Long bizId, @RequestParam(name = "pageNo") Integer pageNo,
			@RequestParam(name = "pageSize") Integer pageSize) {
		Page<CommentVo> page = new Page<CommentVo>();
		page = replyService.getCommentPage(bizId, pageNo, pageSize);
        List<Long> accountIds = new ArrayList<>();
        Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
		for(CommentVo vo:page.getRecords()) {
            if (!accountIds.contains(vo.getAccountId())) {
                accountIds.add(vo.getAccountId());
            }
            if (!accountIds.contains(vo.getReplyId())) {
                accountIds.add(vo.getReplyId());
            }
		}
        List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
        if (CollectionUtils.isNotEmpty(accountVOList)) {
            for (AccountVO accountVO : accountVOList) {
                accountVOMap.put(accountVO.getId(), accountVO);
            }
        }
        //将用户名放入返回结果中
        for (CommentVo commentReplyVO : page.getRecords()) {
            if (null != accountVOMap.get(commentReplyVO.getAccountId())) {
            	AccountVO commentAccount = accountVOMap.get(commentReplyVO.getAccountId());
            	commentReplyVO.setCommentator(commentAccount.getName());
                commentReplyVO.setCommentatorName(commentAccount.getFullName()==""||commentAccount.getFullName()==null?commentAccount.getName():commentAccount.getFullName());
                commentReplyVO.setUserAvatar(accountVOMap.get(commentReplyVO.getAccountId()).getHeadPortrait());
            }
            if (null != accountVOMap.get(commentReplyVO.getReplyId())) {
            	AccountVO replyAccount = accountVOMap.get(commentReplyVO.getReplyId());
                commentReplyVO.setReplyName(replyAccount.getFullName()==""||replyAccount.getFullName()==null?replyAccount.getName():replyAccount.getFullName());
            }
        }
		return page;
	}
	
	/**
	 * 用于学员移动端回复列表查询
	 * @param commmentId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/list2")
	public Page<NewReply> replyList2(@RequestParam("commentId") Long commentId, @RequestParam("pageNo") Integer pageNo,
									   @RequestParam("pageSize") Integer pageSize) {
		RequestContext res = ContextHolder.get();
		Page<NewReply> page = new Page<NewReply>(pageNo, pageSize);
		NewReply reply = new NewReply();
		reply.setCommentId(commentId);
		reply.setState(1);
		EntityWrapper<NewReply> wrapper = new EntityWrapper<NewReply>(reply);
		List<String> list = new ArrayList<String>();
		list.add("createTime");
		wrapper.orderDesc(list);
		replyService.selectPage(page, wrapper);
        //获取评论人和被评论人id并获取信息放入map中
        List<Long> accountIds = new ArrayList<>();
        Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
		for (NewReply listCommentVo : page.getRecords()) {
            if (!accountIds.contains(listCommentVo.getCreateById())) {
                accountIds.add(listCommentVo.getCreateById());
            }
            if (!accountIds.contains(listCommentVo.getCommenterId())) {
                accountIds.add(listCommentVo.getCommenterId());
            }
		}
        List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
        if (CollectionUtils.isNotEmpty(accountVOList)) {
            for (AccountVO accountVO : accountVOList) {
                accountVOMap.put(accountVO.getId(), accountVO);
            }
        }
        //获取敏感词
        Set<String> sensitiveWordList = new HashSet<>();
        String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, res.getCompanyId().toString());
        if (StringUtils.isNotBlank(sensitiveValues)) {
            sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
        }
        //将用户名放入返回结果中
        for (NewReply commentReplyVO : page.getRecords()) {

            //过滤敏感词
            if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                if (null != commentReplyVO.getContent()) {
                    commentReplyVO.setContent(filterWord(commentReplyVO.getContent(), sensitiveWordList));
                }
            }

            if (null != accountVOMap.get(commentReplyVO.getCreateById())) {
            	AccountVO commentAccount = accountVOMap.get(commentReplyVO.getCreateById());
                commentReplyVO.setCreateByName(commentAccount.getFullName()==""||commentAccount.getFullName()==null?commentAccount.getName():commentAccount.getFullName());
                commentReplyVO.setUserAvatar(accountVOMap.get(commentReplyVO.getCreateById()).getHeadPortrait());
            }
            if (null != accountVOMap.get(commentReplyVO.getCommenterId())) {
            	AccountVO replyAccount = accountVOMap.get(commentReplyVO.getCommenterId());
                commentReplyVO.setCommenterName(replyAccount.getFullName()==""||replyAccount.getFullName()==null?replyAccount.getName():replyAccount.getFullName());
            }
        }
		return page;
	}
	
    /**
     * 过滤敏感词
     *
     * @param content           评论内容
     * @param sensitiveWordList 已设置的敏感词集合
     * @return
     */
    public String filterWord(String content, Set<String> sensitiveWordList) {
        if (sensitiveWordList != null) {
            if (!StringUtils.isBlank(content)) {
                return SensitivewordFilter.sensitiveFilter(content, sensitiveWordList);
            }
        }
        return content;
    }
}

